<?php

namespace App\domain\worldpay;

use Illuminate\Support\Facades\Log;

class PaymentGatewayAction
{

    public function execute(array $paymentDetails)
    {
        $response = $this->createCurlRequest();

        Log::info("Worldpay Response: ". $response);

        return $response;
    }

    protected function createCurlRequest(): string|bool
    {
        $curl = curl_init();

        $caBundlePath = base_path('cacert.pem');

        curl_setopt_array($curl, array(
            CURLOPT_URL => "http://dtd.worldpay.com/paymentService_v1.dtd",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS => $this->generateXMLRequest(),
            CURLOPT_HTTPHEADER => array(
                'Content-Type: text/xml; charset=utf-8'
            ),
            CURLOPT_CAINFO => $caBundlePath,
        ));

        $response = curl_exec($curl);

        if (curl_errno($curl)) {
            Log::info('Curl error: ' . curl_error($curl));
        }

        curl_close($curl);

        return $response;
    }

    private function generateXMLRequest(): string
    {
        return '
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE paymentService PUBLIC "-//Worldpay//DTD Worldpay PaymentService v1//EN" "http://dtd.worldpay.com/paymentService_v1.dtd">
<paymentService version="1.4" merchantCode="51016734"> <!--Enter your own merchant code-->
    <submit>
        <order orderCode="CarltonDev0001"> <!--Enter a unique order code each time-->
            <description>YOURDESCRIPTION</description> <!--Enter a description useful to you-->
            <amount currencyCode="GBP" exponent="2" value="0.1" />
            <paymentDetails>
                <CARD-SSL>
                    <cardNumber>4444333322221111</cardNumber>
                    <expiryDate>
                        <date month="01" year="2025" />
                    </expiryDate>
                    <cardHolderName>A Shopper</cardHolderName>
                    <cardAddress>
                        <address>
                            <address1>47A</address1>
                            <address2>Queensbridge Road</address2>
                            <address3>Suburbia</address3>
                            <postalCode>CB94BQ</postalCode>
                            <city>Cambridge</city>
                            <state>Cambridgeshire</state>
                            <countryCode>GB</countryCode>
                        </address>
                    </cardAddress>
                </CARD-SSL>
                <session shopperIPAddress="124.43.240.161" id="0215ui8ib1" />
            </paymentDetails>
            <shopper>
                <shopperEmailAddress>ashopper@myprovider.com</shopperEmailAddress>
                <browser>
                    <acceptHeader>text/html</acceptHeader>
                    <userAgentHeader>Mozilla/5.0 ...</userAgentHeader>
                </browser>
            </shopper>
            <dynamicMCC>5045</dynamicMCC> <!--The merchant category code that applies to this transaction-->
            <dynamicInteractionType type="ECOMMERCE" /> <!--The type of shopper interaction for this transaction-->
            <dynamic3DS overrideAdvice="do3DS" /> <!--Optional override element to ignore preset dynamic 3DSrules-->
        </order>
    </submit>
</paymentService>
';
    }
}
